#include "C:\Program Files\MobileRobots\Aria\include\Aria.h"
#include "tnplib.h"
#include "Init.h"
#include <iostream>
#include <cstring>
#include <fstream>
#include <strstream>
#include <cmath>

using namespace std;

/********************************************************************
** Connection procedures                                           **
********************************************************************/

SOCKET CANStart() {
    SocketLibStart(); // but do it in main() rather than here
    SOCKET s = TCPStartClient("localhost", 8888);
    if (!isValidSocket(s)) return(s);
    TCPSetTimeout(s, 5000); // how long to wait for Matlab in ms
    return(s);
}

const char* CANGet(SOCKET s) {
    const int   bufsize=1400;
    static char buffer[bufsize];
    int status;
    status=TCPSendLine(s, "do\r\n");
    if (status<0) return(0);
    status=TCPRecvLine(s, buffer, bufsize);
    if (status<0) return(0);
    return(buffer);
}

void CANStop(SOCKET s) {
    TCPPrepClose(s);
    TCPStopClient(s);
//  SocketLibEnd();
}

class ConnHandler
{
public:
  // Constructor
  ConnHandler(ArRobot *robot);
  // Destructor, its just empty
  ~ConnHandler(void) {}
  // to be called if the connection was made
  void connected(void);
  // to call if the connection failed
  void connFail(void);
  // to be called if the connection was lost
  void disconnected(void);
protected:
  // robot pointer
  ArRobot *myRobot;
  // the functor callbacks
  ArFunctorC<ConnHandler> myConnectedCB;
  ArFunctorC<ConnHandler> myConnFailCB;
  ArFunctorC<ConnHandler> myDisconnectedCB;
};

ConnHandler::ConnHandler(ArRobot *robot) :
  myConnectedCB(this, &ConnHandler::connected),  
  myConnFailCB(this, &ConnHandler::connFail),
  myDisconnectedCB(this, &ConnHandler::disconnected)

{
  // set the robot poitner
  myRobot = robot;

  // add the callbacks to the robot
  myRobot->addConnectCB(&myConnectedCB, ArListPos::FIRST);
  myRobot->addFailedConnectCB(&myConnFailCB, ArListPos::FIRST);
  myRobot->addDisconnectNormallyCB(&myDisconnectedCB, ArListPos::FIRST);
  myRobot->addDisconnectOnErrorCB(&myDisconnectedCB, ArListPos::FIRST);
}

// just exit if the connection failed
void ConnHandler::connFail(void)
{
  printf("Failed to connect.\n");
  myRobot->stopRunning();
  Aria::shutdown();
  return;
}

// turn on motors, and sonar, and off amigobot sounds, when connected
void ConnHandler::connected(void)
{
  printf("Connected\n");
  myRobot->comInt(ArCommands::SONAR, 1);	//edited from 0
  myRobot->comInt(ArCommands::ENABLE, 1);
  myRobot->comInt(ArCommands::SOUNDTOG, 0);
}

// lost connection, so just exit
void ConnHandler::disconnected(void)
{
  printf("Lost connection\n");
  exit(0);
}



//ArTcpConnection con;          // connection to the robot via TCP/IP
ArSerialConnection con;         // connection to the robot via Serial
ArRobot         robot;          // the robot
ConnHandler     ch(&robot);     // the connection handler from above
ArSonarDevice   sonar;          // sonar device access


/********************************************************************
** Connecting and starting the robot                               **
********************************************************************/

int PioneerStart() {
    Aria::init(Aria::SIGHANDLE_THREAD); // init area with a dedicated signal handling thread

	//con.setPort("pioneer2.bradley.edu", 8101);  // for TCP/IP
    con.setPort("COM1");                        // for serial
    int ret;
    if ((ret = con.open()) != 0)
    {
        std::string str = con.getOpenMessage(ret);
        printf("ERROR: Open failed: %s\n", str.c_str());
        Aria::shutdown();
        return(0);
    }

    // set the robots connection
    robot.setDeviceConnection(&con);

    // try to connect, if we fail, the connection handler should bail
    if (!robot.blockingConnect())
    {
        // this should have been taken care of by the connection handler
        // but just in case
        printf("ERROR: asyncConnect failed because robot is not running in its own thread.\n");
        Aria::shutdown();
        return(0);
    }
    robot.runAsync(true); // run the robot in its own thread, stop if disconnected

    robot.lock();
    robot.comInt(ArCommands::ENABLE, 1);    // enable motors
    robot.unlock();

    return(1);
}

/********************************************************************
** Stopping the robot                                              **
********************************************************************/

int PioneerStop() {
    robot.lock();
    robot.stop();
    robot.unlock();
    Aria::shutdown();
    con.close();
    return(1);
}


/********************************************************************
** The following is the main algorithm for modeMap                 **
********************************************************************/

int main(int argc, char** argv)
{
	PioneerStart();

	Sleep(3000);

	int i = 0;
	int turned = 0;

/********************************************************************
** The following writes to sonarData.txt for map generation.       **
********************************************************************/

  //ofstream out;
  //ofstream outApp( "sonarData.txt", ios::app );

  distanceTraveled=0;
  robot.lock();
  robot.setVel(speed);
  robot.unlock();
  cout<<"start";

  /*out.open("sonarData.txt");
	 if (!out.is_open())
	 {
		  cout<<"Cannot Open File"<<endl;
		  return false;
	  }

 	// Sets up the SonarData.txt file to account for the number and position of sensors
	out<<"robotrun"<<endl;
	out<<"numsonars 8"<<endl;
	out<<"start sonar"<<endl;
	out<<"80 140 90"<<endl;
	out<<"130 125 45"<<endl;
	out<<"155 80 30"<<endl;
	out<<"185 30 15"<<endl;
	out<<"185 -30 -15"<<endl;
	out<<"155 -80 -30"<<endl;
	out<<"130 -125 -45"<<endl;
	out<<"80 -140 -90"<<endl;
	out<<"end"<<endl;
	out<<"robotradius 200"<<endl;
	out<<"DATA SONAR"<<endl;*/
	robot.resetOdometer();

	s = CANStart();

  while(1)
  {

	  Map();

  }

	//out.close();
	PioneerStop();
	return 0;
}
